import fs from 'fs-extra'
import os from 'os'
import ejs from 'ejs'
import readline from 'readline'
import colors from 'colors'
import {
  CLIENT_PATH,
  API_PATH,
  firstLowerCase
} from './../utils/tools'

/**
 * desc:增加一个页面 newPage
 *
 * 1.在api下生成api访问js
 *      -api
 *          -newPage.js
 *
 * 2.在api/index.js增加路由引入
 *
 */
export default async (pageNames) => {
  for (let i = 0, len = pageNames.length; i < len; i++) {
    var pageName = pageNames[i]

    // 1.在api下生成api访问js

    // 首字母小写文件名
    let lowerPageName = firstLowerCase(pageName)

    let apiFile = `${API_PATH}/${lowerPageName}.js`
    let apiTemplateFile = `${CLIENT_PATH}/template/api.js`

    // 读取api文件模版读取api文件模版
    let apiTemplateFileData = await fs.readFile(apiTemplateFile)

    fs.writeFileSync(apiFile, ejs.render(apiTemplateFileData.toString(), {
      lowerPageName
    }))

    // 2.在api/index.js增加路由引入

    // 导入的API
    let importApiStr = `import ${lowerPageName} from './${lowerPageName}'`

    let apiIndexFile = `${API_PATH}/index.js`

    let apiIndexCopyTemplateFile = `${CLIENT_PATH}/template/api-index-copy.js`

    // 同步创建api-index-copy.js
    fs.writeFileSync(apiIndexCopyTemplateFile)

    // 打开apiIndexFile可读的文件流
    let apiIndexReadStream = fs.createReadStream(apiIndexFile)

    // 创建routerCopyFile可写的文件流
    let apiIndexCopyWriteStream = fs.createWriteStream(apiIndexCopyTemplateFile)

    // 创建一个可逐行读取流
    // input - 需要监听的可读流
    let apiFileReadline = readline.createInterface({
      input: apiIndexReadStream
    })

    let index = 0
    await apiFileReadline.on('line', async (line) => {
      let beforeStr = ''
      if (index == 0) {
        index++
      } else {
        beforeStr = os.EOL
      }
      let insertStr = ''
      if (line.indexOf('export default') > -1) {
        insertStr = beforeStr + importApiStr + os.EOL + line + os.EOL + '  ' + lowerPageName + ','
      } else {
        insertStr = beforeStr + line
      }
      await apiIndexCopyWriteStream.write(insertStr)
    })

    await fs.copy(apiIndexCopyTemplateFile, apiIndexFile)
    await fs.unlink(apiIndexCopyTemplateFile)

    console.log(colors.magenta('visit api file :' + apiFile))
  }
}
