import fs from 'fs-extra'
import ip from 'ip'
import os from 'os'
import ejs from 'ejs'
import readline from 'readline'
import colors from 'colors'
import {
  CLIENT_PATH,
  SRC_ROUTER_PATH,
  SRC_VIEW_PATH,
  PROJECT_PATH,
  firstUpperCase,
  firstLowerCase
} from './../utils/tools'

/**
 * desc:增加一个页面 newPage
 *
 * 1.在view下生成页面
 *      -view
 *          -NewPage
 *              -Index.vue
 * 2.在router增加路由引入
 *
 */
// export default pageNames => pageNames.forEach(async pageName => {
//     console.log(pageName);
//     await sleep(3000);
//     console.log(124);
// })
// export default async (pageNames) => {
//     for (let i = 0, len = pageNames.length; i < len; i++) {
//         var pageName = pageNames[i];
//         console.log(pageName);
//         await sleep(3000);
//         console.log(124);
//     }
// }
let devPort = require(PROJECT_PATH + '/package.json').vivo.devPort

export default async (pageNames) => {
  for (let i = 0, len = pageNames.length; i < len; i++) {
    var pageName = pageNames[i]

    // 1.在view下生成页面对应的文件

    // 首字母大写文件名
    let upperPageName = firstUpperCase(pageName)

    // 首字母小写文件名
    let lowerPageName = firstLowerCase(pageName)

    // 新页面
    let newPageFile = `${SRC_VIEW_PATH}/${upperPageName}/Index.vue`

    // 新页面模版
    let newPageTemplateFile = `${CLIENT_PATH}/template/page.vue`

    // 读取模版字符串
    let newPageTemplateData = await fs.readFile(newPageTemplateFile)

    // 生成最终的新页面字符串
    let newPageTemplateStr = await ejs.render(newPageTemplateData.toString(), {
      upperPageName,
      lowerPageName
    })

    // 写入到用户的项目目录中
    await fs.outputFile(newPageFile, newPageTemplateStr)

    // 2.在router增加路由引入

    // 生成router.js
    let routerFile = `${SRC_ROUTER_PATH}/index.js`

    // 拷贝的router.js后的地址
    let routerCopyFile = `${CLIENT_PATH}/template/router-copy.js`

    // 同步创建router-copy.js
    fs.writeFileSync(routerCopyFile)

    // 打开routerFile可读的文件流
    let routerFileReadStream = await fs.createReadStream(routerFile)

    // 创建routerCopyFile可写的文件流
    let routerCopyFileWriteStream = await fs.createWriteStream(routerCopyFile)

    // 创建的路由字符串
    // 导入新建的路由字符串
    let importRouterStr = `const ${upperPageName} = resolve => require(['../views/${upperPageName}/Index'], resolve)`

    // 路由对象字符串
    let routerObjectStr = `
    {
      path: '/${lowerPageName}',
      name: '${upperPageName}',
      meta: {
        title: '${upperPageName}'
      },
      component: ${upperPageName}
    },`

    // 创建一个可逐行读取流
    // input - 需要监听的可读流
    let routerFileReadline = await readline.createInterface({
      input: routerFileReadStream
    })

    // 按行将需要插入的字符串插入至临时文件中
    // 判断是否是首航标示
    let index = 0
    await routerFileReadline.on('line', async (line) => {
      let beforeStr = ''
      // 如果不是首行，则添加系统换行符
      if (index > 0) {
        beforeStr = os.EOL
      }

      // 行数递增
      index++

      // 声明插入的字符
      let insetStr = ''
      if (line.indexOf('new Router') > -1) {
        // 判断到声明路由时，插入importRouterStr
        insetStr = beforeStr + importRouterStr + os.EOL + line
      } else if ((line.replace(/\s/g, '')).indexOf('routes:[') > -1) {
        // 判断路由对象数组时，插入routerObjectStr
        insetStr = beforeStr + line + routerObjectStr
      } else {
        // 其他则还原
        insetStr = beforeStr + line
      }
      await routerCopyFileWriteStream.write(insetStr)
    })
    // 将输出后的router-copy.js拷贝至router/index.js
    await fs.copy(routerCopyFile, routerFile)

    // 追加换行 eslint-eol-last
    await fs.appendFile(routerFile, os.EOL)

    // 删除router-copy.js
    await fs.unlink(routerCopyFile)

    console.log(colors.magenta('visit page file :' + newPageFile))
    console.log(colors.green(`visit page url : http://${ip.address()}:${devPort}/#/${lowerPageName}`))
  }
}
