import fs from 'fs-extra'
import colors from 'colors'
import {
  CLIENT_PATH,
  PROJECT_PATH,
  deleteFolder
} from './../utils/tools'
const exec = require('child_process').execSync
const ora = require('ora')
const getGitUser = require('./../utils/git-user')
const spinner = ora('正在备份工作区代码')

export default async () => {
  spinner.start()
  // 备份用户所有代码
  let copyName = `copy_${+new Date()}`
  await fs.copy(`${PROJECT_PATH}/`, `${CLIENT_PATH}/${copyName}`)
  await fs.copy(`${CLIENT_PATH}/${copyName}`, `${PROJECT_PATH}/${copyName}`)
  spinner.text = '备份完毕'
  spinner.succeed()
  deleteFolder(`${CLIENT_PATH}/${copyName}`)

  // 比对package.json
  // dependencies
  // devDependencies

  exec(`cd ${CLIENT_PATH}/project && git clone -b vue-init ssh://${getGitUser().name}@smartgit:29418/mobileInternet/core/FED/public `)
  let sysProjectPath = `${CLIENT_PATH}/project/public/vue-init/src/project`

  // 读取本地package.json
  let localPackage = fs.readJsonSync(`${sysProjectPath}/package.json`)
  let localDendencies = localPackage.dependencies
  let localDevDendencies = localPackage.devDependencies

  // 读取用户package.json
  let userPackage = fs.readJsonSync(`${PROJECT_PATH}/package.json`)
  let userDendencies = userPackage.dependencies
  let userDevDendencies = userPackage.devDependencies

  var cache = {
    dependencies: {},
    devDependencies: {}
  }

  for (const key in localDendencies) {
    const localVersion = localDendencies[key]
    if (userDendencies.hasOwnProperty(key)) {
      var _userVserion = userDendencies[key]
      // 如果本地版本大于用户版本，则使用本地版本
      if (localVersion > _userVserion) {
        cache.dependencies[key] = `${_userVserion}=>${localVersion}`
        userDendencies[key] = localVersion
      }
    } else {
      cache.dependencies[key] = ` new =>${localVersion}`
      userDendencies[key] = localVersion
    }
  }

  for (const key in localDevDendencies) {
    const localVersion = localDevDendencies[key]
    if (userDevDendencies.hasOwnProperty(key)) {
      var userVserion = userDevDendencies[key]
      // 如果本地版本大于用户版本，则使用本地版本
      if (localVersion > userVserion) {
        cache.devDependencies[key] = `${userVserion}=>${localVersion}`
        userDevDendencies[key] = localVersion
      }
    } else {
      cache.devDependencies[key] = ` new =>${localVersion}`
      userDevDendencies[key] = localVersion
    }
  }

  // 更新scripts
  userPackage.scripts = localPackage.scripts

  let updatePackageFile = `${CLIENT_PATH}/template/update-package.json`
  fs.writeJsonSync(updatePackageFile, userPackage, {
    spaces: '  '
  })

  // 更新package.json
  await fs.copy(updatePackageFile, `${PROJECT_PATH}/package.json`, { overwrite: true }, err => {
    if (err) return console.error(err)
    console.log('本次package.json的依赖的更新：')
    console.log(JSON.stringify(cache, null, '   '))
  })

  // 更新build
  let updateBuildFile = `${sysProjectPath}/build`
  let useBuildFile = `${PROJECT_PATH}/build`
  await fs.remove(useBuildFile)
  await fs.copy(updateBuildFile, useBuildFile).then(() => {
    console.log(`${useBuildFile} 已更新`)
  })

  // 更新gulpfile.js
  let updateGulpFile = `${sysProjectPath}/gulpfile.js`
  let useGulpfile = `${PROJECT_PATH}/gulpfile.js`
  await fs.copy(updateGulpFile, useGulpfile).then(() => {
    console.log(`${useGulpfile} 已更新`)
  })

  // 更新readme
  let updateReadme = `${sysProjectPath}/readme.md`
  let useReadme = `${PROJECT_PATH}/readme.md`
  await fs.copy(updateReadme, useReadme).then(() => {
    console.log(`${useReadme} 已更新`)
  })

  // 更新eslintignore
  let updateEslintignore = `${sysProjectPath}/.eslintignore`
  let useEslintignore = `${PROJECT_PATH}/.eslintignore`
  await fs.copy(updateEslintignore, useEslintignore).then(() => {
    console.log(`${useEslintignore} 已更新`)
  })

  // 更新eslintrc
  let updateEslintrc = `${sysProjectPath}/.eslintrc.js`
  let useEslintrc = `${PROJECT_PATH}/.eslintrc.js`
  await fs.copy(updateEslintrc, useEslintrc).then(() => {
    console.log(`${useEslintrc} 已更新`)
  })

  await fs.remove(`${PROJECT_PATH}/node_modules`)

  console.log(colors.magenta('Project updated successfully !\nPlease run `npm i` '))

  // 移除临时文件夹
  fs.remove(CLIENT_PATH + '/project/public')
}
